import os

import json

from charmhelpers.core.hookenv import (
    local_unit,
    related_units,
    relation_get,
    relation_ids
)

from charmhelpers.core.templating import render

def amqp_ctx():
    ctxs = [ { "rabbit_server": relation_get("hostname", unit, rid),
               "rabbit_user": "contrail",
               "rabbit_password": relation_get("password", unit, rid),
               "rabbit_vhost": "contrail" }
             for rid in relation_ids("amqp")
             for unit in related_units(rid) ]
    return ctxs[0] if ctxs else {}

def cassandra_ctx():
    return { "cassandra_servers": [ relation_get("private-address", unit, rid)
                                    + ":" + port
                                    for rid in relation_ids("cassandra")
                                    for unit, port in
                                    ((unit, relation_get("port", unit, rid)) for unit in related_units(rid))
                                    if port ] }

def contrail_ifmap_ctx():
    creds = []
    unit = local_unit()
    for rid in relation_ids("contrail-ifmap"):
        cs = relation_get("creds", unit, rid)
        if cs:
            cs = json.loads(cs)
            for c in cs.itervalues():
                creds.append(c)
    return { "ifmap_creds": creds }

def fix_permissions():
    os.chmod("/etc/contrail", 0755)
    os.chown("/etc/contrail", 0, 0)

def identity_admin_ctx():
    ctxs = [ { "auth_host": relation_get("service_hostname", unit, rid),
               "auth_port": relation_get("service_port", unit, rid),
               "admin_user": relation_get("service_username", unit, rid),
               "admin_password": relation_get("service_password", unit, rid),
               "admin_tenant_name": relation_get("service_tenant_name", unit, rid) }
             for rid in relation_ids("identity-admin")
             for unit in related_units(rid) ]
    return ctxs[0] if ctxs else {}

def write_contrail_api_config():
    ctx = {}
    ctx.update(cassandra_ctx())
    ctx.update(zookeeper_ctx())
    ctx.update(amqp_ctx())
    ctx.update(identity_admin_ctx())
    render("contrail-api.conf", "/etc/contrail/contrail-api.conf", ctx, "root",
           "contrail", 0440)

def write_contrail_schema_config():
    ctx = {}
    ctx.update(cassandra_ctx())
    ctx.update(zookeeper_ctx())
    ctx.update(identity_admin_ctx())
    render("contrail-schema.conf", "/etc/contrail/contrail-schema.conf", ctx,
           "root", "contrail", 0440)

def write_discovery_config():
    ctx = {}
    ctx.update(zookeeper_ctx())
    ctx.update(cassandra_ctx())
    render("discovery.conf", "/etc/contrail/discovery.conf", ctx)

def write_ifmap_config():
    ctx = contrail_ifmap_ctx()
    render("basicauthusers.properties",
           "/etc/ifmap-server/basicauthusers.properties", ctx, "root",
           "contrail", 0440)

def write_vnc_api_config():
    ctx = identity_admin_ctx()
    render("vnc_api_lib.ini", "/etc/contrail/vnc_api_lib.ini", ctx)

def zookeeper_ctx():
    return { "zk_servers": [ relation_get("private-address", unit, rid)
                             + ":" + port
                             for rid in relation_ids("zookeeper")
                             for unit, port in
                             ((unit, relation_get("port", unit, rid)) for unit in related_units(rid))
                             if port ] }
